pub mod svg;
use svg::*;

use askama::Template;
use derive_more::From;
use lazy_static::lazy_static;
use std::{fs::OpenOptions, io::Write};

#[derive(Template)]
#[template(path = "document.xml")]
struct DocumentTemplate<'a> {
    dimensions: &'a DocumentDimensions,
    keyboard: kle_serial::Keyboard,
}

struct DocumentDimensions {
    width: SVGMeasure,
    height: SVGMeasure,
    margin: SVGMeasure,
}

fn kle_font_units(kle_font_units: &usize) -> SVGMeasure {
    return SVGMeasure::new((6 + kle_font_units * 2) as f64, SVGUnit::Pixel);
}

lazy_static! {
    static ref LETTER_LANDSCAPE: DocumentDimensions = DocumentDimensions {
        width: SVGMeasure::new(11.0, SVGUnit::Inch),
        height: SVGMeasure::new(8.5, SVGUnit::Inch),
        margin: SVGMeasure::new(0.25, SVGUnit::Inch),
    };
}

#[derive(From, Debug)]
enum Error {
    Io(std::io::Error),
    Template(askama::Error),
    LayoutParse(serde_json::Error),
}

fn main() -> Result<(), Error> {
    let mut file = OpenOptions::new()
        .write(true)
        .truncate(true)
        .create(true)
        .open("output.svg")?;
    let document = DocumentTemplate {
        dimensions: &LETTER_LANDSCAPE,
        keyboard: serde_json::from_str(&std::fs::read_to_string("layout.json")?)?,
    };
    write!(file, "{}", document.render()?)?;
    Ok(())
}
